@@ -177,8 +177,8 @@ aws3D <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=TRUE,
     #
     tobj <- .Fortran(C_chaws2,as.double(y),
                      as.double(sigma2),
-                     as.logical(!mask),
-                     as.logical(weighted),
+                     as.integer(!mask),
+                     as.integer(weighted),
                      as.integer(n1),
                      as.integer(n2),
                      as.integer(n3),
@@ -199,8 +199,8 @@ aws3D <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=TRUE,
     if(testprop) {
       pobj <- .Fortran(C_chaws2,as.double(y),
                        as.double(sigma2),
-                       as.logical(!mask),
-                       as.logical(weighted),
+                       as.integer(!mask),
+                       as.integer(weighted),
                        as.integer(n1),
                        as.integer(n2),
                        as.integer(n3),
@@ -255,7 +255,7 @@ aws3D <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=TRUE,
   cat("\nfmri.smooth: first variance estimate","\n")
   vartheta0 <- .Fortran(C_ivar,as.double(residuals),
                            as.double(resscale),
-                           as.logical(mask),
+                           as.integer(mask),
                            as.integer(ddim[1]),
                            as.integer(ddim[2]),
                            as.integer(ddim[3]),
@@ -264,8 +264,8 @@ aws3D <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=TRUE,
   cat("fmri.smooth: smooth the residuals","\n")
   residuals <- .Fortran(C_ihaws2,as.double(residuals),
                      as.double(sigma2),
-                     as.logical(!mask),
-                     as.logical(weighted),
+                     as.integer(!mask),
+                     as.integer(weighted),
                      as.integer(ddim[1]),
                      as.integer(ddim[2]),
                      as.integer(ddim[3]),
@@ -289,7 +289,7 @@ aws3D <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=TRUE,
   cat("fmri.smooth: estimate correlations","\n")
   lags <- pmin(c(5,5,3),ddim[1:3]-1)
   scorr <- .Fortran(C_imcorr,as.double(residuals),
-                     as.logical(mask),
+                     as.integer(mask),
                      as.integer(n1),
                      as.integer(n2),
                      as.integer(n3),
@@ -302,7 +302,7 @@ aws3D <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=TRUE,
   cat("fmri.smooth: final variance estimate","\n")
   vartheta <- .Fortran(C_ivar,as.double(residuals),
                            as.double(resscale),
-                           as.logical(mask),
+                           as.integer(mask),
                            as.integer(n1),
                            as.integer(n2),
                            as.integer(n3),
@@ -457,8 +457,8 @@ aws3Dfull <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=T
     tobj <- .Fortran(C_chawsv,as.double(y),
                      as.double(residuals),
                      as.double(sigma2),
-                     as.logical(!mask),
-                     as.logical(weighted),
+                     as.integer(!mask),
+                     as.integer(weighted),
                      as.integer(n1),
                      as.integer(n2),
                      as.integer(n3),
@@ -487,8 +487,8 @@ aws3Dfull <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=T
       pobj <- .Fortran(C_chawsv,as.double(y),
                         as.double(residuals),
                         as.double(sigma2),
-                        as.logical(!mask),
-                        as.logical(weighted),
+                        as.integer(!mask),
+                        as.integer(weighted),
                         as.integer(n1),
                         as.integer(n2),
                         as.integer(n3),
@@ -540,7 +540,7 @@ aws3Dfull <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=T
   cat("fmri.smooth: estimate correlations","\n")
   lags <- pmin(c(5,5,3),ddim[1:3]-1)
   scorr <- .Fortran(C_imcorr,as.double(tobj$resnew),
-                     as.logical(mask),
+                     as.integer(mask),
                      as.integer(n1),
                      as.integer(n2),
                      as.integer(n3),
@@ -608,8 +608,8 @@ smooth3D <- function(y,lkern="Gaussian",weighted=FALSE,sigma2=NULL,mask=NULL,hma
     ysmooth <- .Fortran(C_smooth3d,
                      as.double(y),
                      as.double(sigma2),
-                     as.logical(!mask),
-                     as.logical(weighted),
+                     as.integer(!mask),
+                     as.integer(weighted),
                      as.integer(n1),
                      as.integer(n2),
                      as.integer(n3),
